# -*- coding: utf-8 -*-
"""
Interaction of classes in `bayesvalidrox`
@author: Rebecca Kohlhaas
"""
import graphviz
from pyUML import Graph, UMLClass

# List all classes and functions outside classes
classes = {'PyLinkForwardModel':{'type_':'Model'}, 
           'BayesInference':{'type_':'Bayes'},  
           'BayesModelComparison':{'type_':'Bayes'}, 
           'Discrepancy':{'type_':'Bayes'}, 
           'MCMC':{'type_':'Bayes'}, 
           'PostProcessing':{'type_':'PostProcessing'}, 
           'BayesLinearRegression':{'type_':'training'}, 
           'EBLinearRegression':{'type_':'training'}, 
           'VBLinearRegression':{'type_':'training'}, 
           'Engine':{'type_':'Engine'}, 
           'ExpDesigns':{'type_':'Design'}, 
           'Exploration':{'type_':'Engine'}, 
           'InputSpace':{'type_':'Design'}, 
           'Input':{'type_':'Design'}, 
           'Marginal':{'type_':'Design'}, 
           'OrthogonalMatchingPursuit':{'type_':'training'}, 
           'RegressionFastARD':{'type_':'training'}, 
           'RegressionFastLaplace':{'type_':'training'}, 
           'MetaModel':{'type_':'MetaModel'}}

free_functions = ['gelman_rubin', 'within_range', 'adaptPlot','apoly_construction',
                  'gamma_mean', 'hellinger_distance', 'logpdf', 'subdomain', 'eval_rec_rule',
                  'eval_rec_rule_arbitrary', 'eval_univ_basis', 'poly_rec_coeffs', 'check_ranges',
                  'cross_truncate', 'glexindex', 'corr', 'update_precisions',
                  'corr_loocv_error', 'create_psi', 'gaussian_process_emulator']

if 0:
    # Create digraph object
    g = graphviz.Digraph('classinteraction', filename='classinteraction.gv', engine='fdp')
    
    # Add the classes and functions as nodes
    g.attr('node', shape='box')
    #for c in classes:
    #    g.node(c)
        
    #g.attr('node', shape='ellipse')
    #for f in free_functions:
    #    g.node(f)
        
    # Create topic-based subclusters
    with g.subgraph(name='cluster_input') as b:
        b.edge('Marginal', 'Input')
    g.edge('cluster_input', 'ExpDesigns')
    g.edge('cluster_input', 'MetaModel')
    
    g.edge('InputSpace', 'MetaModel')
    
    
    
    with g.subgraph(name='cluster_regression') as a:
        for i in classes:
            if classes[i]['type_'] == 'training':
                a.node(i)
    g.edge('cluster_regression', 'MetaModel')
        
    
    g.edge('PyLinkForwardModel', 'Engine')
    g.edge('ExpDesigns', 'Engine')
    g.edge('MetaModel', 'Engine')
    g.edge('Exploration', 'Engine')
    
    with g.subgraph(name='cluster_post') as c:
        c.node('PostProcessing')
        c.edge('BayesInference', 'BayesModelComparison')
        c.edge('MCMC', 'BayesInference')
        
    g.edge('Engine', 'cluster_post')
    g.edge('Discrepancy', 'Engine')
    g.edge('Discrepancy', 'BayesInference')
    # Add the class-class interactions
    if 0:
        g.edge('InputSpace', 'MetaModel')
        g.edge('Engine', 'PostProcessing')
        g.edge('Engine', 'BayesInference')
        g.edge('Engine', 'BayesModelComparison')
    
    g.view()

def print_and_save(graph, name):
    print(graph.to_string())
    graph.write_raw(f"{name}.dot")
    graph.write(f"{name}.png", "dot", "png")

# Create UML for the input classes
graph = Graph('pyUML')

# Add input class
inputs = UMLClass('Input',
                  attributes={
                          'Marginals':'list',
                          'Rosenblatt':'bool'
                      },
                  methods=['add_marginals()']
                  )
graph.add_class(inputs)

# Add marginal class
marginals = UMLClass('Marginal',
                     attributes={
                             'name':'string',
                             'dist_type': 'string',
                             'parameters': 'list',
                             'input_data':'array',
                             'moments': 'list'
                         }
                     )
graph.add_class(marginals)
graph.add_composition(marginals, inputs, multiplicity_parent=1, multiplicity_child='1..*')

inputspace = UMLClass('InputSpace',
                      attributes = {
                          'input_obj':'Input',
                          'meta_Model_type':'string'
                          },
                      methods = [
                          'check_valid_inputs()', 'init_param_space()', 
                          'build_polytypes()', 'transform()'
                          ]
                      )
graph.add_class(inputspace)
graph.add_composition(inputs, inputspace, multiplicity_parent=1, multiplicity_child=1)

expdesign = UMLClass('ExpDesigns',
                     attributes = {
                         'sampling_method': 'string',
                         'hdf5_file': 'string',
                         'n_new_samples': 'int',
                         'n_max_samples': 'int',
                         'mod_LOO_threshold': 'double',
                         'tradeoff_scheme':'string',
                         'n_canddidate': 'int',
                         'explore_method':'string',
                         'exploit_method': 'string',
                         'util_func':'string',
                         'n_cand_groups': 'int',
                         'n_replication': 'int',
                         'post_snapshot': 'bool', 
                         'step_snapshot':'int',
                         'max_a_post': 'list',
                         'adapt_verbose':'bool',
                         'max_func_itr': 'int'
                         },
                     methods = [
                         'generate_samples()', 'generate_ED()', 
                         'read_from_file()', 'random_sampler()',
                         'pcm_sampler()'
                         ])
graph.add_class(expdesign)
graph.add_implementation(expdesign, inputspace)

print_and_save(graph, 'input_classes')